package ObservuPlugins::SystemCpu;

#### globals for cpu stats
# TODO: move these to $self
my @cpu_fields = ( 'user', 'nice','system','idle','iowait','irq','softirq','steal');
my @cpu_state = ();
my @cpu_prev_state = ();

# create a new plugin object
sub new {
    my $class = shift;
    my $self = {
      _config => shift,
      _state => {}
    };
    bless $self, $class;
    return $self;
}


# return a hash with all produced metrics
sub produce_data{
  my($self) = @_;
  
  return read_cpu();  
}



#
#The meanings of the columns are as follows, from left to right:
#
#    * user: normal processes executing in user mode
#    * nice: niced processes executing in user mode
#    * system: processes executing in kernel mode
#    * idle: twiddling thumbs
#    * iowait: waiting for I/O to complete
#    * irq: servicing interrupts
#    * softirq: servicing softirqs 
#    * steal
# 
# From: http://www.linuxhowtos.org/System/procstat.htm
#    
sub read_cpu {
  
  @cpu_prev_state = @cpu_state;  
   
  open(my $f, "<", "/proc/stat");
  if(defined(my $cpuline = <$f>)){
    if($cpuline =~ /^cpu\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s([0-9]+)\s+([0-9]+)(\s+([0-9]+))?/){
      @cpu_state = ($1,$2,$3,$4,$5,$6,$7,trim($8));
    }
  }
  close($f);

  my $result = {};
  if(@cpu_prev_state > 0){

    my @diff = ();
    my $total = 0;    
    for(my $i=0; $i<@cpu_state; $i++){
      $diff[$i] = $cpu_state[$i] - $cpu_prev_state[$i];
      $total += $diff[$i];
    }
    for(my $j=0; $j<@cpu_fields; $j++){
      $result->{"cpu.".$cpu_fields[$j].":percentage"} = 100*$diff[$j]/$total;
    }   
  }
  
  return $result;
  
}

# Perl trim function to remove whitespace from the start and end of the string
sub trim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}
