package ObservuPlugins::SystemDiskIO;

#### globals for disk IO stats
my @diskio_fields = ("reads:Ops", "writes:Ops","read_throughput:Bps","write_throughput:Bps");
my $diskio_state = {};
my $diskio_prev_state = {};
my $diskio_time = 0;
my $diskio_prev_time = 0;

# create a new plugin object
sub new {
    my $class = shift;
    my $self = {
      _config => shift,
      _state => {}
    };
    bless $self, $class;
    return $self;
}


# return a hash with all produced metrics
sub produce_data{
  my($self) = @_;
  
  return read_diskio();  
}



#
# Field 1 -- # of reads issued
# Field 2 -- # of reads merged, field 6 -- # of writes merged
# Field 3 -- # of sectors read
# Field 4 -- # of milliseconds spent reading
# Field 5 -- # of writes completed
# Field 7 -- # of sectors written
# Field 8 -- # of milliseconds spent writing
# Field 9 -- # of I/Os currently in progress
# Field 10 -- # of milliseconds spent doing I/Os
# Field 11 -- weighted # of milliseconds spent doing I/Os 
#
# From: http://www.linuxquestions.org/questions/suse-novell-60/interpreting-proc-diskstats-360350/
# based on http://www.mjmwired.net/kernel/Documentation/iostats.txt
sub read_diskio {
  
  # based on http://cpan.uwinnipeg.ca/htdocs/Sys-Statistics-Linux/Sys/Statistics/Linux/DiskStats.pm.html
  # and http://www.linuxquestions.org/questions/suse-novell-60/interpreting-proc-diskstats-360350/
  # sector size apears to be fixed at 512 bytes, so we will use that for now.
  my $sectorsize = 512;
  
  
  $diskio_prev_time = $diskio_time;

  $diskio_prev_state = $diskio_state;
  $diskio_state = {};
  $diskio_time = time;

  open(my $f, "<", "/proc/diskstats");
  while(defined(my $diskioline = <$f>)){
    if($diskioline =~ /^\s*[0-9]+\s+[0-9]+\s+([0-9a-z]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)/){
      if(not $1 =~/^(ram|loop|nbd)/){ # only real disks
        my @state = ($2, $6, $4*$sectorsize, $7*$sectorsize); 
        # reads, writes
        $diskio_state->{$1} = [@state];
      }
    }
  }
  close($f);

  my $result = {};

  if($diskio_prev_time > 0){

    my $k;
    foreach $k ( keys %{$diskio_state} ){
      my @v = @{$diskio_state->{$k}};
      my @diff = ();
      for(my $i=0; $i<@v; $i++){
        my @vo = @{ $diskio_prev_state->{$k}};
        $diff[$i] = $v[$i] - $vo[$i];
      }
      for(my $j=0; $j<@diskio_fields; $j++){
        my $d = $diff[$j];
        my $dt = $diskio_time - $diskio_prev_time;
        if($dt > 0 && $d >= 0){
          my $ps = $d/$dt;
          if($j>1){
            $ps = int($ps); # don't round  reads/writes per sec
          }
          $result->{"disk_io.".$k.".".$diskio_fields[$j]} = $ps;
        }
      }
    }
  }
  
  return $result;
}

