#
# This will install the observu daemon, this script is execting to run in the 
# same directory as the installation files
#

DAEMON="./observu_daemon.pl"
INSTALL_DIR=`pwd`
DEST_DIR="/usr/local/observu"
INITD_SCRIPT="/etc/init.d/observu"

HELP_TEXT="Installation script options:\n
 -s Skip automated dependency installation\n
 -n No configuration wizard\n
 -m Manual daemon start\n
 -h This help text\n
"


echo ""
echo "Welcome to the Observu Installer"

# Read configuration options
SKIP_AUTO_DEPS=0
NO_CONFIG_WIZARD=0
MANUAL_START=0

while getopts ":snhm" opt; do
  case $opt in
    h)
      echo -e $HELP_TEXT
      exit 1
      ;;
    s)
      SKIP_AUTO_DEPS=1
      ;;
    n)
      NO_CONFIG_WIZARD=1
      ;;
    m)
      MANUAL_START=1
      ;;
  esac
done




# check dependencies
if [ ! `command -v perl` ]
then
  echo "Could not find perl, aborting..."
  exit 1
fi

$DAEMON checkdeps
status=$?
if test $status -ne 0
then
  echo "Dependency check failed..."
  if test $SKIP_AUTO_DEPS -ne 0
    then
      echo "Not trying to fix, remove -s option to change this."
      exit 1
  elif [ `command -v apt-get` ]
    then
      echo "Trying to install libwww-perl and libcrypt-ssleay-perl using apt-get..."
      apt-get install -y --force-yes libwww-perl
      apt-get install -y --force-yes libcrypt-ssleay-perl
  elif [ `command -v yum` ]
    then
      echo "Trying to install perl-libwww-perl and perl-Crypt-SSLeay using yum..."
      yum -y install perl-libwww-perl
      yum -y install perl-Crypt-SSLeay
  else
#    perl -MCPAN -e ‘install Bundle::LWP’
    echo "Could not find an appropriate package manager to fix dependencies, please install libwww for perl manually. Then continue installation by going into the '$INSTALL_DIR' directory and running ./install.sh ";
    exit 1
  fi


  # recheck deps
  $DAEMON checkdeps
  status=$?
  if test $status -ne 0
  then
    echo "Dependencies failed despite our effort, aborting... make sure you run the installer as root (e.g. using sudo) to be able to install the dependencies automatically."
    exit 1  
  fi
  
fi

# create directory to install into
if [ ! -d $DEST_DIR ]
then
  mkdir $DEST_DIR
fi


if [ ! -d $DEST_DIR ]
then
  echo "Unable to create destination directory: '$DEST_DIR' make sure you are running the installer as root (e.g. using sudo). "
  exit 1
fi


# copy scripts to an appropriate location
cp -R ./* $DEST_DIR/
status=$?
if test $status -eq 0
then
  cd $DEST_DIR
else
  echo "Unable to copy program files into $DEST_DIR"
  exit 1
fi

# create init.d script from the skeleton script
cp ./init.d.skeleton $INITD_SCRIPT
chmod 755 $INITD_SCRIPT

# do   update-rc.d /  chkconfig

if [ `command -v update-rc.d` ]
then
  # set up runlevels on debian-like systems
  update-rc.d observu defaults 99
elif [ `command -v chkconfig` ]
then
  # set up runlevels on redhat-like systems
  chkconfig --add observu
  chkconfig --level 2345 observu on
else
  echo "Unfamiliar teritory, I'm not sure how to properly set the runlevels on your system, you will need to do that manually."
  exit 1
fi


# run the daemon  ( start  or install???)

status=0

if test $NO_CONFIG_WIZARD -eq 0
then
  $DAEMON generate_config
  status=$?
fi

if test $MANUAL_START -ne 0
then
  echo "Daemon not started, use /etc/init.d/observu start to do so"
elif test $status -eq 0
then
  $INITD_SCRIPT start
fi

